"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const blessed = require("blessed");
const format_output_1 = require("./format-output");
const PERCENT_MULTIPLIER = 100;
const DEFAULT_SCROLL_OPTIONS = {
    scrollable: true,
    input: true,
    alwaysScroll: true,
    scrollbar: {
        ch: " ",
        inverse: true
    },
    keys: true,
    vi: true,
    mouse: true
};
class Dashboard {
    constructor(title) {
        this.color = "green";
        this.entryMap = {};
        const stdout = process.stdout;
        this.screen = blessed.screen({
            title: title || "EZPack Dashboard",
            smartCSR: true,
            dockBorders: false,
            fullUnicode: true,
            autoPadding: true,
            output: {
                write: stdout.write.bind(stdout),
                isTTY: stdout.isTTY,
                on: stdout.on.bind(stdout),
                removeListener: stdout.removeListener.bind(stdout),
                writable: stdout.writable,
                bytesWritten: stdout.bytesWritten,
                columns: stdout.columns,
                rows: stdout.rows
            }
        });
        stdout.write = this.setLog.bind(this);
        process.stderr.write = stdout.write;
        this.layoutEntry();
        this.layoutStatus();
        this.layoutLog();
        this.screen.key(["escape", "q", "C-c"], () => {
            // eslint-disable-next-line no-process-exit
            process.exit(0);
        });
        this.screen.render();
    }
    mapNavigationKeysToScrollLog() {
        this.screen.key(["pageup"], () => {
            this.logText.setScrollPerc(0);
            this.logText.screen.render();
        });
        this.screen.key(["pagedown"], () => {
            // eslint-disable-next-line no-magic-numbers
            this.logText.setScrollPerc(100);
            this.logText.screen.render();
        });
        this.screen.key(["up"], () => {
            this.logText.scroll(-1);
            this.logText.screen.render();
        });
        this.screen.key(["down"], () => {
            this.logText.scroll(1);
            this.logText.screen.render();
        });
    }
    layoutEntry() {
        this.entry = blessed.box({
            label: "Entry",
            padding: 1,
            width: "75%",
            height: "36%",
            left: "0%",
            top: "0%",
            border: {
                type: "line"
            },
            style: {
                fg: -1,
                border: {
                    fg: this.color
                }
            }
        });
        this.entryText = blessed.log(Object.assign({}, DEFAULT_SCROLL_OPTIONS, {
            parent: this.entry,
            tags: true,
            width: "100%-5"
        }));
        this.screen.append(this.entry);
    }
    layoutLog() {
        this.log = blessed.box({
            label: "Log",
            padding: 1,
            width: "100%",
            height: "66%",
            left: "0%",
            top: "36%",
            border: {
                type: "line"
            },
            style: {
                fg: -1,
                border: {
                    fg: this.color
                }
            }
        });
        this.logText = blessed.log(Object.assign({}, DEFAULT_SCROLL_OPTIONS, {
            parent: this.log,
            tags: true,
            width: "100%-5"
        }));
        this.screen.append(this.log);
        this.mapNavigationKeysToScrollLog();
    }
    layoutStatus() {
        this.wrapper = blessed.layout({
            width: "25%",
            height: "36%",
            top: "0%",
            left: "75%",
            layout: "grid"
        });
        this.status = blessed.box({
            parent: this.wrapper,
            label: "Status",
            tags: true,
            padding: {
                left: 1
            },
            width: "100%",
            height: "34%",
            valign: "middle",
            border: {
                type: "line"
            },
            style: {
                fg: -1,
                border: {
                    fg: this.color
                }
            }
        });
        this.operations = blessed.box({
            parent: this.wrapper,
            label: "Operation",
            tags: true,
            padding: {
                left: 1
            },
            width: "100%",
            height: "34%",
            valign: "middle",
            border: {
                type: "line"
            },
            style: {
                fg: -1,
                border: {
                    fg: this.color
                }
            }
        });
        this.progress = blessed.box({
            parent: this.wrapper,
            label: "Progress",
            tags: true,
            padding: 1,
            width: "100%",
            height: "34%",
            valign: "middle",
            border: {
                type: "line"
            },
            style: {
                fg: -1,
                border: {
                    fg: this.color
                }
            }
        });
        this.progressbar = new blessed.ProgressBar({
            parent: this.progress,
            height: 1,
            width: "90%",
            top: "center",
            left: "center",
            orientation: "horizontal",
            style: {
                bar: {
                    bg: this.color
                }
            }
        });
        this.screen.append(this.wrapper);
    }
    setProgress(val) {
        const percent = parseFloat(val) * PERCENT_MULTIPLIER;
        const formattedPercent = `${percent.toFixed(2)}%`;
        if (!percent) {
            this.progressbar.setProgress(percent);
        }
        this.progressbar.setContent(formattedPercent);
        this.progressbar.setProgress(percent);
    }
    setOperations(val) {
        this.operations.setContent(val);
    }
    setStatus(val) {
        let content;
        switch (val) {
            case "Success":
                content = `{green-fg}{bold}${val}{/}`;
                break;
            case "Failed":
                content = `{red-fg}{bold}${val}{/}`;
                break;
            default:
                content = `{bold}${val}{/}`;
        }
        this.status.setContent(content);
    }
    setStats(data) {
        if (data.hasErrors()) {
            this.status.setContent("Failed");
        }
        this.logText.log(format_output_1.formatOutput(data));
    }
    todo(name) {
        console.log(`TODO: ${name}`);
    }
    setLog(val) {
        this.logText.log(val.replace(/[{}]/g, ""));
    }
    setEntry(entry) {
        if (!this.entryMap[entry]) {
            this.entryText.log(`{yellow-fg}${entry}{/}`);
            this.entryMap[entry] = true;
        }
    }
    clear() {
        this.logText.setContent("");
    }
    setData(dataArray) {
        const actionForMessageType = data => {
            const map = {
                progress: this.setProgress.bind(this),
                operations: this.setOperations.bind(this),
                status: this.setStatus.bind(this),
                stats: this.setStats.bind(this),
                nodeEnv: this.todo.bind(this, "nodeEnv"),
                log: this.setLog.bind(this),
                entry: this.setEntry.bind(this),
                clear: this.clear.bind(this),
                sizes: this.todo.bind(this, "sizes"),
                problems: this.todo.bind(this, "problems")
            };
            return map[data.type](data.value);
        };
        dataArray.forEach(actionForMessageType);
        this.screen.render();
    }
}
exports.Dashboard = Dashboard;
let dashboard;
function getDashboard(title) {
    if (!dashboard) {
        dashboard = new Dashboard(title);
    }
    return dashboard;
}
exports.getDashboard = getDashboard;
//# sourceMappingURL=index.js.map