"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const CacheData_1 = require("../CacheData");
const config_1 = require("../config");
const webpackDevMiddleware = require("webpack-dev-middleware");
const express = require("express");
const ezmok = require("ezmok");
const dashboard_1 = require("./dashboard");
const RouteEntryPlugin_1 = require("./RouteEntryPlugin");
const RouteEntry_1 = require("./RouteEntry");
const host = "0.0.0.0";
function getRandomKey(keyList) {
    const randomIndex = Math.floor(Math.random() * keyList.length);
    return keyList[randomIndex];
}
function genInitEntry(originEntries, serverConfig) {
    const initEntry = {};
    function addToInitEntry(entryKey) {
        if (originEntries[entryKey] && !initEntry[entryKey]) {
            initEntry[entryKey] = originEntries[entryKey];
        }
    }
    if (serverConfig.initEntry) {
        serverConfig.initEntry.forEach(addToInitEntry);
    }
    const cacheEntry = CacheData_1.devEntryCache.getEntry();
    if (cacheEntry.length > 0) {
        cacheEntry.forEach(addToInitEntry);
    }
    // if no entry, generate random one
    if (Object.keys(initEntry).length === 0) {
        const k = getRandomKey(Object.keys(originEntries));
        addToInitEntry(k);
    }
    return initEntry;
}
function checkEntryForRoute(check) {
    return function ensurePage(req, res, next) {
        const requestPath = req.path === "/" ? "/index.html" : req.path;
        const url = requestPath.split(".");
        if (url[1] === "html") {
            const entryName = url[0].substr(1);
            check(entryName, next);
        }
        else {
            next();
        }
    };
}
function setExpressStaticDir(app, serverConfig) {
    const { contentBase, staticOptions } = serverConfig;
    if (Array.isArray(contentBase)) {
        contentBase.forEach(root => {
            if (typeof root === "string") {
                app.use(express.static(root, staticOptions));
            }
            else {
                app.use(root.prefix, express.static(root.path, staticOptions));
            }
        });
    }
    else if (contentBase) {
        if (typeof contentBase === "string") {
            app.use(express.static(contentBase, staticOptions));
        }
        else {
            app.use(contentBase.prefix, express.static(contentBase.path, staticOptions));
        }
    }
}
function startServer(webpackConfig, serverConfig) {
    const originEntries = webpackConfig.entry;
    const dashboard = dashboard_1.getDashboard(serverConfig.name);
    const initEntry = genInitEntry(originEntries, serverConfig);
    webpackConfig.entry = initEntry;
    const compiler = webpack(webpackConfig);
    const devMiddleware = webpackDevMiddleware(compiler, {
        logLevel: "silent",
        headers: {
            "Access-Control-Allow-Origin": "*"
        }
    });
    const port = serverConfig.port;
    const localHost = `http://localhost:${port}`;
    const printEntry = (entryName) => dashboard.setEntry(`${localHost}/${entryName}.html`);
    const routeEntry = new RouteEntry_1.RouteEntry({
        initEntry: Object.keys(initEntry),
        devMiddleware,
        allEntries: originEntries,
        afterEntryBuild: printEntry
    });
    new RouteEntryPlugin_1.RouteEntryPlugin({
        getEntriesForBuild: routeEntry.getEntriesForBuild,
        done: routeEntry.finishBuild
    }).apply(compiler);
    const app = express();
    // init express middleware
    if (serverConfig.defaultProxyOrigin && serverConfig.apiDefPath) {
        app.use(ezmok({
            defaultApiOrigin: serverConfig.defaultProxyOrigin,
            defPath: serverConfig.apiDefPath,
            mockDirectory: config_1.HOME_DIR
        }));
    }
    app.use(checkEntryForRoute(routeEntry.checkRoute));
    app.use(devMiddleware);
    app.use(require("webpack-hot-middleware")(compiler, {
        // tslint:disable-next-line:no-empty
        log: () => { }
    }));
    setExpressStaticDir(app, serverConfig);
    app.locals.env = process.env.NODE_ENV;
    dashboard.setEntry(`${localHost}/ezmok`);
    devMiddleware.waitUntilValid(() => {
        Object.keys(webpackConfig.entry).forEach(k => {
            printEntry(k);
        });
    });
    console.log("> Starting dev server...");
    app.listen(port, host);
}
exports.default = startServer;
//# sourceMappingURL=index.js.map