"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ejs = require("ejs");
const fs = require("fs");
const html_minifier_1 = require("html-minifier");
const path = require("path");
const config_1 = require("../../config");
const utils_1 = require("../../utils");
function getAssetName(rawName) {
    let name = rawName;
    if (name.includes("/")) {
        let list = name.split("/");
        name = list[list.length - 1];
    }
    return name.split(".")[0];
}
function readFileContent(fileName) {
    return fs.readFileSync(fileName).toString();
}
function insertPreload({ rel, asValue, includes }, chunks, htmlContent, entryName) {
    let preloadChunks = [];
    chunks = chunks.filter(chunk => !chunk.files[0].endsWith(".map"));
    if (includes === "all" || (typeof includes === "object" && includes[entryName])) {
        const entryChunk = chunks.find(chunk => chunk.initial && chunk.names[0] === entryName);
        if (typeof entryChunk !== "undefined") {
            chunks = chunks.filter(chunk => chunk.parents.includes(entryChunk.id));
        }
        else {
            chunks = [];
        }
        if (includes !== "all") {
            includes = includes[entryName];
        }
    }
    if (includes === "all") {
        preloadChunks = chunks.map(chunk => ({
            name: chunk.names[0],
            file: chunk.files[0]
        }));
    }
    else if (Array.isArray(includes)) {
        preloadChunks = chunks
            .map(chunk => ({
            name: chunk.names[0],
            file: chunk.files[0]
        }))
            .filter(chunk => includes.includes(chunk.name));
    }
    if (preloadChunks.length === 0) {
        return htmlContent;
    }
    const appendHtml = preloadChunks
        .map(({ file, name }) => {
        if (rel === "preload") {
            if (typeof asValue === "undefined") {
                if (file.endsWith(".css")) {
                    asValue = "style";
                }
                else if (file.endsWith("woff2")) {
                    asValue = "font";
                }
                else {
                    asValue = "script";
                }
            }
            const crossOrigin = asValue === "font" ? `crossorigin="crossorigin" ` : "";
            return `<link rel="preload" as="${asValue}" href=${file} ${crossOrigin}>`;
        }
        return `<link rel="${rel}" href=${file} >`;
    })
        .join("\n");
    return htmlContent.replace("</head>", appendHtml + "</head>");
}
class HtmlPlugin {
    constructor(options) {
        this.options = Object.assign({
            template: path.resolve(config_1.ROOT_PATH, "../config/index.html"),
            title: "ezbuy"
        }, options);
    }
    apply(compiler) {
        compiler.plugin("emit", (compilation, cb) => {
            const { entry, template, entryTemplates, preload, disableMinify } = this.options;
            const { assets, chunks } = compilation.getStats().toJson();
            const allAssets = chunks
                .filter(chunk => chunk.initial)
                .map(chunk => ({
                name: chunk.names[0],
                file: chunk.files[0],
                origins: chunk.origins
            }));
            const entryAssets = [];
            const entryDepsMap = {};
            allAssets.forEach(({ file, origins, name }) => {
                const assetFileName = getAssetName(file);
                if (entry[assetFileName]) {
                    entryAssets.push({
                        name: assetFileName,
                        content: file
                    });
                }
                else {
                    if (Array.isArray(origins)) {
                        origins.forEach(({ loc }) => {
                            if (!Array.isArray(entryDepsMap[loc])) {
                                entryDepsMap[loc] = [];
                            }
                            entryDepsMap[loc].push({
                                name,
                                file
                            });
                        });
                    }
                }
            });
            // get dll entry
            const dllAssets = assets
                .filter(asset => asset.name.endsWith(".dll.js"))
                .map(asset => asset.name)
                .sort((a) => (a.startsWith("common") ? -1 : 1));
            const shareStyles = assets
                .filter(asset => asset.name.includes(".css"))
                .map(asset => asset.name);
            let defaultTemplateContent = "";
            if (template) {
                defaultTemplateContent = readFileContent(template);
            }
            else {
                // 正常的逻辑不会进入这里
                utils_1.printError("The default template is empty.");
                return;
            }
            const scripts = entryAssets.map(item => {
                const templateContent = typeof entryTemplates !== "undefined" && typeof entryTemplates[item.name] !== "undefined"
                    ? readFileContent(entryTemplates[item.name])
                    : defaultTemplateContent;
                const entryDeps = Array.isArray(entryDepsMap[item.name]) ? entryDepsMap[item.name] : [];
                const runtimeDeps = entryDeps.find(({ name }) => name.startsWith("runtime~"));
                const scripts = entryDeps
                    .filter(({ name }) => !name.startsWith("runtime~"))
                    .map(({ file }) => file);
                const manifestContent = runtimeDeps ? compilation.assets[runtimeDeps.file].source() : "";
                if (runtimeDeps) {
                    delete compilation.assets[runtimeDeps.file];
                }
                scripts.push(...dllAssets, item.content);
                let htmlContent = ejs.render(templateContent, {
                    title: this.options.title,
                    styles: shareStyles,
                    scripts,
                    manifestContent,
                    customTplData: this.options.customTplData,
                    options: this.options
                });
                if (this.options.isProd && typeof preload !== "undefined") {
                    htmlContent = insertPreload(preload, chunks, htmlContent, item.name);
                }
                return {
                    name: item.name,
                    content: disableMinify !== true && this.options.isProd
                        ? html_minifier_1.minify(htmlContent, {
                            removeAttributeQuotes: true,
                            removeComments: true,
                            collapseWhitespace: true
                        })
                        : htmlContent
                };
            });
            scripts.forEach(item => {
                compilation.assets[`${item.name}.html`] = {
                    source: () => item.content,
                    size: () => item.content.length
                };
            });
            return cb();
        });
    }
}
exports.default = HtmlPlugin;
//# sourceMappingURL=HtmlPlugin.js.map