import { EventEmitter } from "events";
import { devEntryCache } from "../CacheData";
import { printWarn } from "../utils";
import { EntryForBuild } from "./RouteEntryPlugin";

export interface CheckRoute {
	(entryName: string, done: () => void): void;
}

export interface AfterEntryBuild {
	(entryName: string): void;
}

export enum EntryStatus {
	ADDED,
	BUILDING,
	BUILT
}

export interface BuiltEntry {
	[index: string]: EntryStatus;
}

export interface AllEntries {
	[entryName: string]: string;
}

export interface RouteEntryOptions {
	afterEntryBuild: AfterEntryBuild;
	initEntry: string[];
	devMiddleware: any;
	// ezpack config 里面配置的所有 entry
	allEntries: AllEntries;
}

export class Invalidator {
	private devMiddleware: any;
	private building: boolean;
	private rebuildAgain: boolean;

	constructor(devMiddleware) {
		this.devMiddleware = devMiddleware;
		this.building = false;
		this.rebuildAgain = false;
	}

	invalidate() {
		if (this.building) {
			this.rebuildAgain = true;
			return;
		}

		this.building = true;
		this.devMiddleware.invalidate();
	}

	startBuilding() {
		this.building = true;
	}

	doneBuilding() {
		this.building = false;
		if (this.rebuildAgain) {
			this.rebuildAgain = false;
			this.invalidate();
		}
	}
}

export class RouteEntry {
	private builtEntries: BuiltEntry;
	private invalidator: Invalidator;
	private allEntries: AllEntries;
	private doneCallbacks: EventEmitter;
	private afterEntryBuild: AfterEntryBuild;

	constructor(o: RouteEntryOptions) {
		this.builtEntries = {};
		if (o.initEntry.length > 0) {
			o.initEntry.forEach(k => {
				this.builtEntries[k] = EntryStatus.BUILDING;
			});
		}

		this.invalidator = new Invalidator(o.devMiddleware);
		this.allEntries = o.allEntries;
		this.doneCallbacks = new EventEmitter();
		this.afterEntryBuild = o.afterEntryBuild;

		this.getEntriesForBuild = this.getEntriesForBuild.bind(this);
	}

	private addEntry = (name: string) => {
		devEntryCache.addEntry(name);
		this.builtEntries[name] = EntryStatus.ADDED;
	};

	private setBuilding() {
		this.invalidator.startBuilding();
		Object.keys(this.builtEntries).forEach(k => {
			this.builtEntries[k] = EntryStatus.BUILDING;
		});
	}

	startBuild = () => {
		// 这里的顺序不能调换，必须先 invalidate 再 setBuilding
		this.invalidator.invalidate();
		this.setBuilding();
	};

	finishBuild = () => {
		Object.keys(this.builtEntries).forEach(page => {
			const status = this.builtEntries[page];
			if (status !== EntryStatus.BUILDING) return;

			this.builtEntries[page] = EntryStatus.BUILT;
			this.doneCallbacks.emit(page);
			this.invalidator.doneBuilding();
			this.afterEntryBuild(page);
		});
	};

	getEntriesForBuild(): EntryForBuild[] {
		return Object.keys(this.builtEntries).map(name => ({
			name,
			entry: this.allEntries[name]
		}));
	}

	checkRoute: CheckRoute = (entryName: string, done: () => void) => {
		if (!this.allEntries[entryName]) {
			done();
			return;
		}

		const status = this.builtEntries[entryName];
		if (status) {
			if (status === EntryStatus.BUILT) {
				done();
			} else if (status === EntryStatus.BUILDING) {
				this.doneCallbacks.on(entryName, done);
			}
		} else {
			printWarn(`> Building page: ${entryName}`);
			this.addEntry(entryName);
			this.doneCallbacks.on(entryName, done);
			this.startBuild();
		}
	};
}
