import * as blessed from "blessed";
import { formatOutput } from "./format-output";
import * as webpack from "webpack";

const PERCENT_MULTIPLIER = 100;
const DEFAULT_SCROLL_OPTIONS = {
	scrollable: true,
	input: true,
	alwaysScroll: true,
	scrollbar: {
		ch: " ",
		inverse: true
	},
	keys: true,
	vi: true,
	mouse: true
};

export type MessageType = "progress" | "operations" | "status" | "log" | "entry";

export interface MessageHandler {
	[messageType: string]: () => void;
}

export interface Data {
	type: MessageType;
	value: any;
}

export interface ActionData {
	value: string;
}

export class Dashboard {
	screen: any;
	progressbar: any;
	log: any;
	logText: any;
	color: string = "green";
	wrapper: any;
	status: any;
	operations: any;
	progress: any;
	entry: any;
	entryText: any;
	entryMap: { [entry: string]: boolean } = {};

	constructor(title?: string) {
		const stdout = process.stdout;

		this.screen = blessed.screen({
			title: title || "EZPack Dashboard",
			smartCSR: true,
			dockBorders: false,
			fullUnicode: true,
			autoPadding: true,
			output: {
				write: stdout.write.bind(stdout),
				isTTY: stdout.isTTY,
				on: stdout.on.bind(stdout),
				removeListener: stdout.removeListener.bind(stdout),
				writable: stdout.writable,
				bytesWritten: (stdout as any).bytesWritten,
				columns: (stdout as any).columns,
				rows: (stdout as any).rows
			}
		});

		stdout.write = this.setLog.bind(this);
		process.stderr.write = stdout.write;

		this.layoutEntry();
		this.layoutStatus();
		this.layoutLog();

		this.screen.key(["escape", "q", "C-c"], () => {
			// eslint-disable-next-line no-process-exit
			process.exit(0);
		});

		this.screen.render();
	}

	mapNavigationKeysToScrollLog() {
		this.screen.key(["pageup"], () => {
			this.logText.setScrollPerc(0);
			this.logText.screen.render();
		});
		this.screen.key(["pagedown"], () => {
			// eslint-disable-next-line no-magic-numbers
			this.logText.setScrollPerc(100);
			this.logText.screen.render();
		});
		this.screen.key(["up"], () => {
			this.logText.scroll(-1);
			this.logText.screen.render();
		});
		this.screen.key(["down"], () => {
			this.logText.scroll(1);
			this.logText.screen.render();
		});
	}

	layoutEntry() {
		this.entry = blessed.box({
			label: "Entry",
			padding: 1,
			width: "75%",
			height: "36%",
			left: "0%",
			top: "0%",
			border: {
				type: "line"
			},
			style: {
				fg: -1,
				border: {
					fg: this.color
				}
			}
		});
		this.entryText = blessed.log(
			Object.assign({}, DEFAULT_SCROLL_OPTIONS, {
				parent: this.entry,
				tags: true,
				width: "100%-5"
			})
		);
		this.screen.append(this.entry);
	}

	layoutLog() {
		this.log = blessed.box({
			label: "Log",
			padding: 1,
			width: "100%",
			height: "66%",
			left: "0%",
			top: "36%",
			border: {
				type: "line"
			},
			style: {
				fg: -1,
				border: {
					fg: this.color
				}
			}
		});

		this.logText = blessed.log(
			Object.assign({}, DEFAULT_SCROLL_OPTIONS, {
				parent: this.log,
				tags: true,
				width: "100%-5"
			})
		);

		this.screen.append(this.log);
		this.mapNavigationKeysToScrollLog();
	}

	layoutStatus() {
		this.wrapper = blessed.layout({
			width: "25%",
			height: "36%",
			top: "0%",
			left: "75%",
			layout: "grid"
		});

		this.status = blessed.box({
			parent: this.wrapper,
			label: "Status",
			tags: true,
			padding: {
				left: 1
			},
			width: "100%",
			height: "34%",
			valign: "middle",
			border: {
				type: "line"
			},
			style: {
				fg: -1,
				border: {
					fg: this.color
				}
			}
		});

		this.operations = blessed.box({
			parent: this.wrapper,
			label: "Operation",
			tags: true,
			padding: {
				left: 1
			},
			width: "100%",
			height: "34%",
			valign: "middle",
			border: {
				type: "line"
			},
			style: {
				fg: -1,
				border: {
					fg: this.color
				}
			}
		});

		this.progress = blessed.box({
			parent: this.wrapper,
			label: "Progress",
			tags: true,
			padding: 1,
			width: "100%",
			height: "34%",
			valign: "middle",
			border: {
				type: "line"
			},
			style: {
				fg: -1,
				border: {
					fg: this.color
				}
			}
		});

		this.progressbar = new blessed.ProgressBar({
			parent: this.progress,
			height: 1,
			width: "90%",
			top: "center",
			left: "center",
			orientation: "horizontal",
			style: {
				bar: {
					bg: this.color
				}
			}
		});

		this.screen.append(this.wrapper);
	}

	setProgress(val: string) {
		const percent = parseFloat(val) * PERCENT_MULTIPLIER;
		const formattedPercent = `${percent.toFixed(2)}%`;

		if (!percent) {
			this.progressbar.setProgress(percent);
		}

		this.progressbar.setContent(formattedPercent);
		this.progressbar.setProgress(percent);
	}

	setOperations(val: string) {
		this.operations.setContent(val);
	}

	setStatus(val: string) {
		let content;

		switch (val) {
			case "Success":
				content = `{green-fg}{bold}${val}{/}`;
				break;
			case "Failed":
				content = `{red-fg}{bold}${val}{/}`;
				break;
			default:
				content = `{bold}${val}{/}`;
		}

		this.status.setContent(content);
	}

	setStats(data: webpack.Stats) {
		if (data.hasErrors()) {
			this.status.setContent("Failed");
		}

		this.logText.log(formatOutput(data));
	}

	todo(name: string) {
		console.log(`TODO: ${name}`);
	}

	setLog(val: string) {
		this.logText.log(val.replace(/[{}]/g, ""));
	}

	setEntry(entry: string) {
		if (!this.entryMap[entry]) {
			this.entryText.log(`{yellow-fg}${entry}{/}`);
			this.entryMap[entry] = true;
		}
	}

	clear() {
		this.logText.setContent("");
	}

	setData(dataArray: Data[]) {
		const actionForMessageType = data => {
			const map = {
				progress: this.setProgress.bind(this),
				operations: this.setOperations.bind(this),
				status: this.setStatus.bind(this),
				stats: this.setStats.bind(this),
				nodeEnv: this.todo.bind(this, "nodeEnv"),
				log: this.setLog.bind(this),
				entry: this.setEntry.bind(this),
				clear: this.clear.bind(this),
				sizes: this.todo.bind(this, "sizes"),
				problems: this.todo.bind(this, "problems")
			};
			return map[data.type](data.value);
		};

		dataArray.forEach(actionForMessageType);

		this.screen.render();
	}
}

let dashboard: Dashboard;

export function getDashboard(title?: string) {
	if (!dashboard) {
		dashboard = new Dashboard(title);
	}

	return dashboard;
}
