import * as webpack from "webpack";
import { devEntryCache } from "../CacheData";
import { HOME_DIR } from "../config";
import * as webpackDevMiddleware from "webpack-dev-middleware";
import * as express from "express";
import * as ezmok from "ezmok";
import * as serveStatic from "serve-static";
import { getDashboard } from "./dashboard";
import { RouteEntryPlugin } from "./RouteEntryPlugin";
import { RouteEntry, CheckRoute } from "./RouteEntry";

const host = "0.0.0.0";

function getRandomKey(keyList: string[]): string {
	const randomIndex = Math.floor(Math.random() * keyList.length);
	return keyList[randomIndex];
}

export interface ServerStaticPath {
	prefix: string;
	path: string;
}

export interface DevServerOption {
	/**
	 * 开发服务器的端口
	 */
	port: number;
	/**
	 * 开发模式下默认的接口请求地址
	 */
	defaultProxyOrigin: ezmok.OriginSetting;
	/**
	 * 接口定义文件根目录路径
	 */
	apiDefPath: string;
	/**
	 * 开发模式初始化编译的 entry
	 */
	initEntry?: string[];
	/**
	 * 开发服务器的静态文件目录
	 */
	contentBase?: string | string[] | ServerStaticPath | ServerStaticPath[];
	/**
	 * 静态目录相关配置
	 */
	staticOptions?: serveStatic.ServeStaticOptions;
	name?: string;
}

function genInitEntry(originEntries: any, serverConfig: DevServerOption) {
	const initEntry = {};

	function addToInitEntry(entryKey: string) {
		if (originEntries[entryKey] && !initEntry[entryKey]) {
			initEntry[entryKey] = originEntries[entryKey];
		}
	}

	if (serverConfig.initEntry) {
		serverConfig.initEntry.forEach(addToInitEntry);
	}

	const cacheEntry = devEntryCache.getEntry();
	if (cacheEntry.length > 0) {
		cacheEntry.forEach(addToInitEntry);
	}

	// if no entry, generate random one
	if (Object.keys(initEntry).length === 0) {
		const k = getRandomKey(Object.keys(originEntries));
		addToInitEntry(k);
	}

	return initEntry;
}

function checkEntryForRoute(check: CheckRoute) {
	return function ensurePage(req, res, next) {
		const requestPath = req.path === "/" ? "/index.html" : req.path;
		const url: string[] = requestPath.split(".");
		if (url[1] === "html") {
			const entryName = url[0].substr(1);
			check(entryName, next);
		} else {
			next();
		}
	};
}

function setExpressStaticDir(app: express.Express, serverConfig: DevServerOption) {
	const { contentBase, staticOptions } = serverConfig;
	if (Array.isArray(contentBase)) {
		(contentBase as any[]).forEach(root => {
			if (typeof root === "string") {
				app.use(express.static(root, staticOptions));
			} else {
				app.use(root.prefix, express.static(root.path, staticOptions));
			}
		});
	} else if (contentBase) {
		if (typeof contentBase === "string") {
			app.use(express.static(contentBase, staticOptions));
		} else {
			app.use(contentBase.prefix, express.static(contentBase.path, staticOptions));
		}
	}
}

export default function startServer(
	webpackConfig: webpack.Configuration,
	serverConfig: DevServerOption
) {
	const originEntries = webpackConfig.entry as any;

	const dashboard = getDashboard(serverConfig.name);

	const initEntry = genInitEntry(originEntries, serverConfig);
	webpackConfig.entry = initEntry;

	const compiler = webpack(webpackConfig);
	const devMiddleware = webpackDevMiddleware(compiler, {
		logLevel: "silent",
		headers: {
			"Access-Control-Allow-Origin": "*"
		}
	});

	const port = serverConfig.port;
	const localHost = `http://localhost:${port}`;
	const printEntry = (entryName: string) => dashboard.setEntry(`${localHost}/${entryName}.html`);

	const routeEntry = new RouteEntry({
		initEntry: Object.keys(initEntry),
		devMiddleware,
		allEntries: originEntries,
		afterEntryBuild: printEntry
	});
	new RouteEntryPlugin({
		getEntriesForBuild: routeEntry.getEntriesForBuild,
		done: routeEntry.finishBuild
	}).apply(compiler);

	const app = express();

	// init express middleware
	if (serverConfig.defaultProxyOrigin && serverConfig.apiDefPath) {
		app.use(
			ezmok({
				defaultApiOrigin: serverConfig.defaultProxyOrigin,
				defPath: serverConfig.apiDefPath,
				mockDirectory: HOME_DIR
			})
		);
	}
	app.use(checkEntryForRoute(routeEntry.checkRoute));
	app.use(devMiddleware);
	app.use(
		require("webpack-hot-middleware")(compiler, {
			// tslint:disable-next-line:no-empty
			log: () => {}
		})
	);
	setExpressStaticDir(app, serverConfig);

	app.locals.env = process.env.NODE_ENV;

	dashboard.setEntry(`${localHost}/ezmok`);
	devMiddleware.waitUntilValid(() => {
		Object.keys(webpackConfig.entry as any).forEach(k => {
			printEntry(k);
		});
	});
	console.log("> Starting dev server...");

	app.listen(port, host);
}
