import * as CopyWebpackPlugin from "copy-webpack-plugin";
import * as webpack from "webpack";
import { BundleAnalyzerPlugin } from "webpack-bundle-analyzer";
import * as merge from "webpack-merge";

import { DefineGlobalPlugin } from "./Plugin/DefineGlobalPlugin";

import { HASH_LEN, HOME_DIR } from "../config";
import genUglifyJsPlugin from "./Plugins/genUglifyJsPlugin";
import { genConfig } from "./base.config";
import { WebpackOptions } from "./interface";

export function genProdConfig(
	options: WebpackOptions,
	name: string,
	isOnline: boolean,
	sentryRelease: string,
	analyse?: boolean
) {
	let { cdnPath, splitChunks } = options;
	if (typeof cdnPath === "object") {
		if (isOnline) {
			cdnPath = cdnPath.online;
		} else {
			cdnPath = cdnPath.uat;
		}
	}
	options.cdnPath = cdnPath;

	const baseConfig = genConfig(options, name, true);
	const sourceMap = !!options.sourceMap;
	const optimization: { minimizer: any[]; splitChunks?: any; runtimeChunk?: boolean } = {
		splitChunks,
		minimizer: [genUglifyJsPlugin(sourceMap, HOME_DIR)]
	};
	const plugins = [
		new webpack.DefinePlugin({
			"process.env.NODE_ENV": JSON.stringify("production")
		}),
		// remove moment locale, except en and zh
		// see https://github.com/moment/moment/issues/2416#issuecomment-344840418
		new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/),
		new webpack.HashedModuleIdsPlugin(),
		new DefineGlobalPlugin({
			SENTRY_RELEASE: sentryRelease
		})
	];
	if (splitChunks === undefined) {
		optimization.splitChunks = {
			chunks: "initial",
			minChunks: 2,
			cacheGroups: {
				commons: {
					name: "common",
					chunks: "initial",
					minChunks: 4
				}
			}
		};
	}
	optimization.runtimeChunk = true;
	if (analyse) {
		plugins.push(
			new BundleAnalyzerPlugin({
				analyzerMode: "static"
			})
		);
	} else {
		// exclude ModuleConcatenationPlugin in analyse mode
		// see https://github.com/webpack-contrib/webpack-bundle-analyzer#i-cant-see-all-the-dependencies-in-a-chunk
		plugins.push(new webpack.optimize.ModuleConcatenationPlugin());
	}

	if (Array.isArray(options.copyPath) && options.cdnPath.length > 0) {
		plugins.push(new CopyWebpackPlugin(options.copyPath));
	}

	const version = typeof options.version === "string" ? `.${options.version}` : "";
	const nameTPL = `js/[name]${version}.[chunkhash:${HASH_LEN}]`;

	return merge(baseConfig, {
		output: {
			path: options.outputPath,
			publicPath: cdnPath,
			filename: `${nameTPL}.js`,
			chunkFilename: `${nameTPL}.chunk.js`
		},
		devtool: sourceMap ? "source-map" : false,
		plugins,
		optimization
	});
}
